#include <Arduino.h>
#include <Ultrasonic.h>

//// Adafruit Display
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);
#define NUMFLAKES 10
#define XPOS 0
#define YPOS 1
#define DELTAY 2

#if (SSD1306_LCDHEIGHT != 64)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

const int PIN_ENA = 0;
const int PIN_ENB = 1;
const int PIN_IN1 = 3; 
const int PIN_IN2 = 5;
const int PIN_IN3 = 9;
const int PIN_IN4 = 10; 

const int SERVO_PIN = 7; 

Ultrasonic ultrasonic(A2,A3);
int distance = 21;
int motorPower = 255;

  
// put your setup code here, to run once:

void setup() {
  Serial.begin(9600);
  pinMode(PIN_ENA,OUTPUT); 
  pinMode(PIN_ENB, OUTPUT);
  pinMode(PIN_IN1, OUTPUT);
  pinMode(PIN_IN2, OUTPUT);
  pinMode(PIN_IN3, OUTPUT);
  pinMode(PIN_IN4, OUTPUT);
  pinMode(SERVO_PIN, OUTPUT);
  digitalWrite(PIN_ENA, HIGH);
  digitalWrite(PIN_ENB, HIGH);

  // Display Initialisieren
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println("2WD China Bots");
  //display.setTextSize(3);
  display.println("HTWK-");
  display.setTextSize(2);
  display.println("Leipzig");
  display.display();
}

#if __GNUC__ && __AVR__
/**
 * @param leftMotor -255 bis 255, Stärke
 * @param rightMotor -255 bis 255, Stärke
 */
void setMotors(int leftMotor, int rightMotor) {
  if (rightMotor>=0)
  {
    analogWrite(PIN_IN2, rightMotor);
    digitalWrite(PIN_IN1, LOW);
  } else {
    analogWrite(PIN_IN2, rightMotor);
    digitalWrite(PIN_IN1, HIGH);
  }

  if (leftMotor>=0)
  {
    analogWrite(PIN_IN4, leftMotor);
    digitalWrite(PIN_IN3, LOW);
  } else {
    analogWrite(PIN_IN4, leftMotor);
    digitalWrite(PIN_IN3, HIGH);
  }
}

void stopMotors() {
  digitalWrite(PIN_IN1,LOW);
  digitalWrite(PIN_IN2,LOW);
  digitalWrite(PIN_IN3,LOW);
  digitalWrite(PIN_IN4,LOW);
  digitalWrite(PIN_ENA,LOW);
  digitalWrite(PIN_ENB,LOW);
}

void enableHigh() {
  digitalWrite(PIN_ENA,HIGH);
  digitalWrite(PIN_ENB,HIGH);
}



void setServo(int degrees) {
  int lenMicroSecondsOfPulse = (float) degrees/ 90.0 * 1000 + 300;
  int lenMicroSecondsOfPeriod = 20 * 1000; // 20 milliseconds (ms) 
  for (int i = 0; i < 200; ++i) {
    digitalWrite(SERVO_PIN, HIGH); 
    // Delay for the length of the pulse
    delayMicroseconds(lenMicroSecondsOfPulse);

    // Turn the voltage low for the remainder of the pulse
    digitalWrite(SERVO_PIN, LOW);

    // Delay this loop for the remainder of the period so we don't
    // send the next signal too soon or too late
    delayMicroseconds(lenMicroSecondsOfPeriod - lenMicroSecondsOfPulse); 

  }

}

int scanDirection(int angle) {
  setServo(angle);
  delay(200);
  return ultrasonic.distanceRead();
}

void setDisplaytext(String text, Adafruit_SSD1306 display) {
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println(text);
  display.display();
}

int lookLeft(){
  setServo(180);
  delay(1000);
  return ultrasonic.distanceRead();
}

int lookRight(){
  setServo(0);
  delay(1000);
  return ultrasonic.distanceRead();
}

#endif

int getX(int b, int alpha) {
  if (alpha<=90)
  {
    return cos(alpha)*b;
  } else
  {
    return cos(alpha)*b;
  }
  
}

int getY(int b, int alpha) {
  return sin(alpha)*b;
}

void loop() {
  char s[50];
  for (int i = 0; i <= 180; i+=1)
  {
    int distance = scanDirection(i);
    int x = getX(distance, i);
    int y = getY(distance, i);
    sprintf(s,"X: %d, Y: %d, b: %d, alpha: %d\n",x,y,distance,i);
    Serial.println(s);
    setDisplaytext(s, display);
  }
}




#ifndef __AVR__

int main()
{
 setup();
 while(1){loop();}
}

int scanDirection(int angle) {
  return 20;
}

int setDisplaytext(String text, Adafruit_SSD1306 display) {
  std::cout<<text;
}
#endif